var Shared_worker = new Class
({
	initialize: function(script, handlers)
	{
		var worker = new SharedWorker(script)
			
		var channel = Shared_worker.channel(worker.port)
		
		worker.port.addEventListener('message', function(event) 
		{
			var type = event.data.type
			var message = event.data.message
			
			Object.for_each(handlers, function(message_type, handler)
			{
				if (type === message_type)
					handler.bind(channel)(message)
			})
		}, 
		false)
		
		worker.port.start()
		
		this.worker = worker
		
		this.send = channel.send
	},
	
	broadcast: function(type, message)
	{
		this.send('broadcast', { type: type, message: message })
	}
})

Shared_worker.new = function(worker, handlers)
{
	var peers = []
	
	worker.addEventListener('connect', function(event) 
	{
		var port = event.ports[0]
		
		peers.push(port)
		
		var context = Shared_worker.channel(port)
		context.id = peers.length
		
		context.broadcast = function(type, message)
		{
			peers.forEach(function(peer) 
			{
				Shared_worker.channel(peer).send(type, message)
			})
		}
		
		context.broadcast_others = function(type, message)
		{
			peers.forEach(function(peer) 
			{
				if (peer === port)
					return
					
				Shared_worker.channel(peer).send(type, message)
			})
		}
		
		port.addEventListener('message', function(event) 
		{
			var type = event.data.type
			var message = event.data.message
			
			if (type === 'broadcast')
				return context.broadcast(message.type, message.message)
			
			Object.for_each(handlers, function(message_type, handler)
			{
				if (type === message_type)
					handler.bind(context)(message)
			})
		})
		
		port.start()
	})
}

Shared_worker.channel = function(port)
{
	var channel =
	{
		send: function(type, message)
		{
			port.postMessage({ type: type, message: message })
		},
		
		reply: function(type, message)
		{
			this.send(type, message)
		}
	}
	
	return channel
}